class ApplicationController < ActionController::Base
  protect_from_forgery
  layout :devise_layout if MagicNumber['test_layout']
  before_filter :bf
  
  def devise_layout
    if devise_controller?
      case resource_name
      when :admin
        'guest'
      when :user
        'guest'
      when :demand_user
        'guest'
      end
    else
      'application'
    end
  end
  
  def bf
    @server_result = {
      :location => {:controller => params[:controller], :action => params[:action]}
    }
    @server_result[:location][:id] = params[:id] if params[:id]
    if Admin.count.to_i == 0 or License.count.to_i == 0
      if params[:controller] == 'system' and params[:action] == 'start'
      else
        redirect_to :controller => '/system', :action => 'start'
      end
    else
      if user_signed_in?
        @user = current_user
        @author = @user.author
        @artist = if @author and @author.artist?
          @author.artist
        else
          nil
        end
      end
      @admin = if admin_signed_in?
        current_admin
      else
        nil
      end
      @demand_user = if demand_user_signed_in?
        current_demand_user
      else
        nil
      end
    end
  end
  
  def authenticate_reader
    authenticate_user! unless (@user || @admin)
  end
  
  def authenticate_user
    authenticate_user! unless (@user)
  end
  
  def authenticate_resource_reader
    authenticate_user! unless (@user || @admin || @demand_user)
  end
  
  def authenticate_author
    if @author
      true
    else
      respond_to do |format|
        format.html { redirect_to main_app.new_author_path, :status => :found }
        format.js { render "authors/new" }
        format.json { 
          raise ActiveRecord::Forbidden
        }
      end
      false
    end
  end
      
  def authenticate_artist
    if @admin or (@author and @author.artist?)
      true
    else
      respond_to do |format|
        format.html { redirect_to main_app.new_artist_path, :status => :found }
        format.js { render "artists/new" }
        format.json { render :text => {:error => I18n.t('devise.failure.unauthenticated')}.to_json, :status => :unauthorized }
      end
      false
    end
  end
  
  def set_image(file)
    if file.respond_to?(:read)
      file.read
    else
      Base64.decode64(file.to_s.gsub(' ', '+'))
    end
  end
  
  if Rails.env == 'production'
    rescue_from ActiveRecord::RecordNotFound, :with => :render_404
    rescue_from ActiveRecord::Forbidden, :with => :render_403
    
    private
    def render_404(exception = nil)
      if exception
        logger.info "Rendering 404: #{exception.message}"
      end
      respond_to do |format|
        format.html { 
          render :file => "#{Rails.root}/public/404.html", :status => 404, :layout => false
        }
        format.json { 
          render :text => "404 Not found", :status => 404
        }
      end
    end
    
    def render_403(exception = nil)
      if exception
        logger.info "Rendering 403: #{exception.message}"
      end
      respond_to do |format|
        format.html { 
          render :file => "#{Rails.root}/public/403.html", :status => 404, :layout => false
        }
        format.json { 
          render :text => "403 Forbidden", :status => 403
        }
      end
    end
  end
end
