class DemandersController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  before_filter :authenticate_demand_user!, :only => [:index, :show, :new, :create, :edit, :update, :destroy, :req]
  
  def index
    @demander = @demand_user.demander
    if @demander
      respond_to do |format|
        format.html # index.html.erb
      end
    else
      flash[:notice] = I18n.t('demanders.flash.notice.not_found')
      redirect_to new_demander_path
    end
  end

  def show
    @demander = Demander.show(params[:id], @demand_user)

    respond_to do |format|
      format.html # show.html.erb
      format.json { render :json => @demander.to_json(Demander.show_json_opt) }
    end
  end

  def new
    @demander = Demander.new
    @demander.supply_default
    respond_to do |format|
      format.html # new.html.erb
      format.json { render json: @demander.to_json(Demander.show_json_opt) }
    end
  end

  def edit
    @demander = Demander.edit(params[:id], @demand_user)
    respond_to do |format|
      format.html 
    end
  end

  def create
    @demander = Demander.new
    @demander.supply_default 
    @demander.attributes = params[:demander]
    @demander.overwrite @demand_user

    respond_to do |format|
      if @demander.store
        flash[:notice] = I18n.t('flash.notice.created', :model => Demander.model_name.human)
        format.html { redirect_to @demander }
        format.json { render json: @demander.to_json(Demander.show_json_opt), status: :created, location: @demander }
      else
        format.html { render action: "new" }
        format.json { render json: @demander.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @demander = Demander.edit(params[:id], @demand_user)
    @demander.attributes = params[:demander]
    @demander.overwrite @demand_user
    respond_to do |format|
      if @demander.store
        flash[:notice] = I18n.t('flash.notice.updated', :model => Demander.model_name.human)
        format.html { redirect_to @demander }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @demander.errors, status: :unprocessable_entity }
      end
    end
  end

  def req
    @demander = @demand_user.demander
    respond_to do |format|
      if @demander.demander_status.request
        flash[:notice] = I18n.t('demanders.flash.notice.requested')
        format.html { redirect_to demanders_path }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('demanders.flash.notice.not_requested')
        format.html { redirect_to demanders_path }
        format.json { render json: @demander.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @demander = Demander.edit(params[:id], @demand_user)
    respond_to do |format|
      if @demander.stop
        flash[:notice] = I18n.t('demanders.flash.notice.stopped')
        format.html { redirect_to demanders_url }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('demanders.flash.notice.not_stopped')
        format.html { redirect_to demanders_url }
        format.json { render json: @demander.errors, status: :unprocessable_entity }
      end
    end
  end

end
