class Balloon < ActiveRecord::Base
  belongs_to :speech_balloon
  belongs_to :system_picture
  
  validates :speech_balloon_id, :numericality => {:allow_blank => true}
  validates :system_picture_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
#  validates :caption, :presence => true
#  validates :settings, :presence => true

  def url
    '/system_pictures/' + self.system_picture.filename
  end
  
  def visible? au
    if au == nil
      return false if MagicNumber['run_mode'] == 1
    elsif au.is_a?(Author)
    elsif au.is_a?(Admin)
      return true
    else
      return false
    end
    self.speech_balloon.visible? au
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(Balloon.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => 'panels.publish > 0', :order => 'balloons.updated_at desc'})
    Balloon.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:speech_balloon => {:panel => {:author => {}}, :speeches => {}, :speech_balloon_template => {} }}}
  end
  
  def self.list_json_opt
    {:include => {:speech_balloon => {:include => {:panel => {:include => {:author => {} }}, :speeches => {}, :speech_balloon_template => {} }}}}
  end
  
  def self.show cid, au
    opt = {}
    opt.merge!(Balloon.show_opt)
    res = Balloon.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(au)
    res
  end
  
  def self.show_opt
    {:include => {:speech_balloon => {:panel => {:author => {}}, :speeches => {}, :speech_balloon_template => {} }}}
  end
  
  def self.show_json_opt
    {:include => {:speech_balloon => {:include => {:panel => {:include => {:author => {} }}, :speeches => {}, :speech_balloon_template => {} }}}}
  end
  
end
