# -*- encoding: utf-8 -*-
require 'spec_helper'
#借手

describe DemandersController do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = @user.author    #ユーザ作成時に連動して作成される
    @demand_user = FactoryGirl.create :demand_user
  end
  
  describe '借手メニューに於いて' do
    before do
      sign_in @demand_user
    end
    context 'つつがなく終わるとき' do
      before do
        @ds = FactoryGirl.create :demander_status
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
      end
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success
      end
      it '借手アカウントから@demanderに借手を取得している' do
        get :index
        assigns(:demander).should eq(@demander)
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
      end
    end
    context '借手が存在しなかったとき' do
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'プロフィール登録ページへ遷移する' do
          get :index
          response.body.should redirect_to new_demander_path
        end
      end
    end
    context '借手権限がないとき' do
      before do
        sign_out @demand_user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.body.should redirect_to '/demand_users/sign_in'
        end
      end
      context 'json形式' do
      end
    end
  end
  
  describe '新規作成フォーム表示に於いて' do
    before do
      sign_in @demand_user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :new
        response.should be_success 
      end
      it '@demanderに新規データを用意している' do
        get :new
        assigns(:demander).should be_a_new(Demander)
      end
      it '借手モデルにデフォルト値補充を依頼している' do
        Demander.any_instance.should_receive(:supply_default).exactly(1)
        get :new
      end
      context 'html形式' do
        it 'newテンプレートを描画する' do
          get :new
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :new, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '借手モデルにjson単体出力オプションを問い合わせている' do
          Demander.should_receive(:show_json_opt).exactly(1)
          get :new, :format => :json
        end
      end
    end
    context '借手権限がないとき' do
      before do
        sign_out @demand_user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :new
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :new
          response.body.should redirect_to '/demand_users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :new, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :new, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '新規作成に於いて' do
    before do
      sign_in @demand_user
      @attr = FactoryGirl.attributes_for(:demander, :demand_user_id => @demand_user.id, :caption => 'normal')
    end
    context '事前チェックしておく' do
      it '借手モデルにデフォルト値補充を依頼している' do
        Demander.any_instance.should_receive(:supply_default).exactly(1)
        post :create, :demander => @attr
      end
      it '借手モデルにカラム値復元を依頼している' do
        Demander.any_instance.should_receive(:attributes=).exactly(1)
        post :create, :demander => @attr
      end
      it '借手モデルに上書き補充を依頼している' do
        Demander.any_instance.should_receive(:overwrite).exactly(1)
        post :create, :demander => @attr
      end
      it 'モデルに保存依頼する' do
        Demander.any_instance.should_receive(:store).exactly(1)
        post :create, :demander => @attr
      end
    end
    context 'つつがなく終わるとき' do
      it "@demanderに作成された借手を保持していて、それがDBにある" do
        post :create, :demander => @attr
        assigns(:demander).should be_a(Demander)
        assigns(:demander).should be_persisted
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          Demander.any_instance.stub(:store).and_return(true)
          post :create, :demander => @attr
          response.status.should eq 302
        end
        it '作成された借手の表示ページへ遷移する' do
#          Demander.any_instance.stub(:save).and_return(true)
          post :create, :demander => @attr
          response.should redirect_to(Demander.last)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          Demander.any_instance.stub(:save).and_return(true)
          post :create, :demander => @attr, :format => :json
          response.should be_success 
        end
        it '作成された借手をjsonデータで返す' do
          post :create, :demander => @attr, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          post :create, :demander => @attr, :format => :json
          json = JSON.parse response.body
          json["caption"].should match(/normal/)
        end
      end
    end
    context '借手権限がないとき' do
      before do
        sign_out @demand_user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :create, :demander => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          post :create, :demander => @attr
          response.body.should redirect_to '/demand_users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          post :create, :demander => @attr, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          post :create, :demander => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗した' do
      before do
        Demander.any_instance.stub(:store).and_return(false)
      end
      it "未保存の借手を保持している" do
        post :create, :demander => @attr
        assigns(:demander).should be_a_new(Demander)
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          post :create, :demander => @attr
          response.status.should eq 200
        end
        it '新規ページを描画する' do
          post :create, :demander => @attr
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          post :create, :demander => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          post :create, :demander => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

  describe '編集フォーム表示に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
      sign_in @demand_user
      Demander.stub(:edit).and_return(@demander)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :edit, :id => @demander.id
        response.should be_success 
      end
      it '借手モデルに編集取得を問い合わせている' do
        Demander.should_receive(:edit).exactly(1)
        get :edit, :id => @demander.id
      end
      it '@demanderにデータを用意している' do
        get :edit, :id => @demander.id
        assigns(:demander).should eq @demander
      end
      context 'html形式' do
        it 'editテンプレートを描画する' do
          get :edit, :id => @demander.id
          response.should render_template("edit")
        end
      end
    end
    context '借手権限がないとき' do
      before do
        sign_out @demand_user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @demander.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :edit, :id => @demander.id
          response.body.should redirect_to '/demand_users/sign_in'
        end
      end
    end
  end

  describe '更新に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
      @attr = FactoryGirl.attributes_for(:demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id, :caption => 'updated caption')
      sign_in @demand_user
    end
    context '事前チェックしておく' do
      it '借手モデルに編集取得を問い合わせている' do
        Demander.stub(:edit).with(any_args()).and_return @demander
        Demander.should_receive(:edit).exactly(1)
        put :update, :id => @demander.id, :demander => @attr
      end
      it '借手モデルにカラム値復元を依頼している' do
        Demander.any_instance.should_receive(:attributes=).exactly(1)
        put :update, :id => @demander.id, :demander => @attr
      end
      it '借手モデルに上書き補充を依頼している' do
        Demander.any_instance.should_receive(:overwrite).exactly(1)
        put :update, :id => @demander.id, :demander => @attr
      end
      it 'モデルに更新を依頼する' do
        Demander.any_instance.stub(:store).with(any_args).and_return true
        Demander.any_instance.should_receive(:store).exactly(1)
        put :update, :id => @demander.id, :demander => @attr
      end
      it '@demanderにアレを取得している' do
        put :update, :id => @demander.id, :demander => @attr
        assigns(:demander).id.should eq(@demander.id)
      end
    end
    context 'つつがなく終わるとき' do
      it '更新される' do
        put :update, :id => @demander.id, :demander => @attr
        Demander.find(@demander.id).caption.should match /updated/
      end
      context 'html形式' do
        before do
          Demander.any_instance.stub(:store).with(any_args()).and_return(true)
        end
        it 'ステータスコード302 Foundを返す' do
          put :update, :id => @demander.id, :demander => @attr
          response.status.should eq 302
        end
        it '更新された借手の表示ページへ遷移する' do
          put :update, :id => @demander.id, :demander => @attr
          response.should redirect_to(@demander)
        end
      end
      context 'json形式' do
        before do
          Demander.any_instance.stub(:store).with(any_args()).and_return(true)
        end
        it 'ステータスコード200 OKを返す' do
          put :update, :id => @demander.id, :demander => @attr, :format => :json
          response.should be_success 
        end
        it 'ページ本体は特に返さない' do
          put :update, :id => @demander.id, :demander => @attr, :format => :json
          response.body.should match /./
        end
      end
    end
    context '借手権限がないとき' do
      before do
        sign_out @demand_user
      end
      it 'ステータスコード302 Foundを返す' do
        put :update, :id => @demander.id, :demander => @attr
        response.status.should eq 302
      end
      context 'html形式' do
        it 'サインインページへ遷移する' do
          put :update, :id => @demander.id, :demander => @attr
          response.body.should redirect_to '/demand_users/sign_in'
        end
      end
      context 'json形式' do
        it '応答メッセージにUnauthorizedを返す' do
          put :update, :id => @demander.id, :demander => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗したとき' do
      before do
        Demander.any_instance.stub(:store).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード200 Okを返す' do
          put :update, :id => @demander.id, :demander => @attr
          response.status.should eq 200
        end
        it '編集ページを描画する' do
          put :update, :id => @demander.id, :demander => @attr
          response.should render_template("edit")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          Demander.any_instance.stub(:save).and_return(false)
          put :update, :id => @demander.id, :demander => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          put :update, :id => @demander.id, :demander => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end
  
  describe '借受申請に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
      @attr = FactoryGirl.attributes_for(:demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id, :caption => 'updated caption')
      sign_in @demand_user
    end
    context '事前チェックしておく' do
      it '借手モデルの貸与状況に借受申請を依頼している' do
        DemanderStatus.stub(:request).with(any_args()).and_return true
        DemanderStatus.any_instance.should_receive(:request).exactly(1)
        post :req
      end
    end
    context 'つつがなく終わるとき' do
      context 'html形式' do
        before do
          DemanderStatus.any_instance.stub(:request).with(any_args()).and_return(true)
        end
        it 'ステータスコード302 Foundを返す' do
          post :req
          response.status.should eq 302
        end
        it '借手ホーム(index)ページへ遷移する' do
          post :req
          response.should redirect_to(demanders_path)
        end
        it '一時的メッセージに処理成功を設定する' do
          post :req
          flash[:notice].should eq I18n.t('demanders.flash.notice.requested')
        end
      end
      context 'json形式' do
        before do
          DemanderStatus.any_instance.stub(:request).with(any_args()).and_return(true)
        end
        it 'ステータスコード200 OKを返す' do
          post :req, :format => :json
          response.should be_success 
        end
        it 'ページ本体は特に返さない' do
          post :req, :format => :json
          response.body.should match /./
        end
      end
    end
    context '借手権限がないとき' do
      before do
        sign_out @demand_user
      end
      it 'ステータスコード302 Foundを返す' do
        post :req
        response.status.should eq 302
      end
      context 'html形式' do
        it 'サインインページへ遷移する' do
          post :req
          response.body.should redirect_to '/demand_users/sign_in'
        end
      end
      context 'json形式' do
        it '応答メッセージにUnauthorizedを返す' do
          post :req, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗したとき' do
      before do
        DemanderStatus.any_instance.stub(:request).with(any_args()).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :req
          response.status.should eq 302
        end
        it '借手ホーム(index)ページへ遷移する' do
          post :req
          response.should redirect_to(demanders_path)
        end
        it '一時的メッセージに処理失敗を設定する' do
          post :req
          flash[:notice].should eq I18n.t('demanders.flash.notice.not_requested')
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          post :req, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          post :req, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end
  
  describe '借受停止に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => Time.now
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      sign_in @demand_user
    end
    context '事前チェックしておく' do
      it '借手モデルに編集取得を問い合わせている' do
        Demander.stub(:edit).with(any_args()).and_return @demander
        Demander.should_receive(:edit).exactly(1)
        delete :destroy, :id => @demander.id
      end
      it '借手モデルに借受停止を依頼する' do
        Demander.any_instance.stub(:stop).with(any_args).and_return true
        Demander.any_instance.should_receive(:stop).exactly(1)
        delete :destroy, :id => @demander.id
      end
      it '@demanderにアレを取得している' do
        delete :destroy, :id => @demander.id
        assigns(:demander).should eq(@demander)
      end
    end
    context 'つつがなく終わるとき' do
      before do
        Demander.any_instance.stub(:stop).with(any_args()).and_return(true)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @demander.id
          response.status.should eq 302
        end
        it '借手ホーム(index)ページに遷移する' do
          delete :destroy, :id => @demander.id
          response.should redirect_to(demanders_path)
        end
        it '一時的メッセージに処理成功を設定する' do
          delete :destroy, :id => @demander.id
          flash[:notice].should eq I18n.t('demanders.flash.notice.stopped')
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          delete :destroy, :id => @demander.id, :format => :json
          response.should be_success 
        end
        it 'ページ本体は特に返さない' do
          delete :destroy, :id => @demander.id, :format => :json
          response.body.should match /./
        end
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @demand_user
      end
      it 'ステータスコード302 Foundを返す' do
        delete :destroy, :id => @demander.id
        response.status.should eq 302
      end
      context 'html形式' do
        it 'サインインページへ遷移する' do
          delete :destroy, :id => @demander.id
          response.body.should redirect_to '/demand_users/sign_in'
        end
      end
      context 'json形式' do
        it '応答メッセージにUnauthorizedを返す' do
          delete :destroy, :id => @demander.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '借受停止に失敗したとき' do
      before do
        Demander.any_instance.stub(:stop).with(any_args).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @demander.id
          response.status.should eq 302
        end
        it '借手ホーム(index)ページに遷移する' do
          delete :destroy, :id => @demander.id
          response.should redirect_to(demanders_path)
        end
        it '一時的メッセージに処理失敗を設定する' do
          delete :destroy, :id => @demander.id
          flash[:notice].should eq I18n.t('demanders.flash.notice.not_stopped')
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          delete :destroy, :id => @demander.id, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          delete :destroy, :id => @demander.id, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

end
