# -*- encoding: utf-8 -*-
#借受状況
require 'spec_helper'

describe ProviderStatus do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = @user.author    #ユーザ作成時に連動して作成される
  end
  describe '検証に於いて' do
    before do
      @ps = FactoryGirl.build :provider_status
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @ps.receive_hour1 = -99999
        @ps.receive_hour2 = -99999
        @ps.should be_valid
      end
      it '上限データが通る' do
        @ps.receive_hour1 = 99999
        @ps.receive_hour2 = 99999
        @ps.should be_valid
      end
    end
    
    context 'receive_hour1を検証するとき' do
      it '数値でなければ失敗する' do
        @ps.receive_hour1 = 'a'
        @ps.should_not be_valid
      end
    end
    context 'receive_hour2を検証するとき' do
      it '数値でなければ失敗する' do
        @ps.receive_hour2 = 'a'
        @ps.should_not be_valid
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'defined' do
      @ps = FactoryGirl.build :provider_status
      @ps.supply_default
    end
  end
  
  describe '上書き補充に於いて' do
    it 'defined' do
      @ps = FactoryGirl.build :provider_status
      @ps.overwrite
    end
  end
  
  describe '所持判定に於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
    end
    it '管理者ならyes' do
      @ps.own?(@admin).should == true
    end
    it '作家ならno' do
      @ps.own?(@author).should == false
    end
    it 'パラメータが管理者でないならno' do
      @ps.own?(nil).should == false
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
    end
    it '管理者なら許可する' do
      r = @ps.visible?(@admin)
      r.should == true
    end
    it '作家なら許可しない' do
      r = @ps.visible?(@author)
      r.should == false
    end
    it 'それ以外のとき不許可を返す。' do
      r = @ps.visible?(nil)
      r.should be_false
    end
  end
  
  describe '状態に於いて' do
    before do
    end
    it '借り受けしていないのとき0を返す' do
      @ps = FactoryGirl.create :provider_status, :token => nil
      r = @ps.status
      r.should eq 0
    end
    it '借り受けしているのとき1を返す' do
      @ps = FactoryGirl.create :provider_status, :token => 'a'
      r = @ps.status
      r.should eq 1
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
      @pd = FactoryGirl.create :provider, :provider_status_id => @ps.id, :name => "6"
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        ProviderStatus.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        ProviderStatus.page().should eq 1
      end
      it '0以下の場合は1になる' do
        ProviderStatus.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        ProviderStatus.page_size('7').should eq 7
      end
      it 'nilの場合はProviderStatus.default_page_sizeになる' do
        ProviderStatus.page_size().should eq ProviderStatus.default_page_size
      end
      it '0以下の場合はProviderStatus.default_page_sizeになる' do
        ProviderStatus.page_size('0').should eq ProviderStatus.default_page_size
      end
      it 'ProviderStatus.max_page_sizeを超えた場合はProviderStatus.max_page_sizeになる' do
        ProviderStatus.page_size('1000').should eq ProviderStatus.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        ProviderStatus.stub(:list_opt).with(any_args).and_return({:include => {:provider => {}} })
        ProviderStatus.should_receive(:list_opt).with(any_args).exactly(1)
        r = ProviderStatus.list
      end
    end
    it 'リストを返す' do
      r = ProviderStatus.list
      r.should eq [@ps]
    end
    it '管理名で並んでいる' do
      v = FactoryGirl.create :provider_status
      vd = FactoryGirl.create :provider, :provider_status_id => v.id, :name => "0"
      r = ProviderStatus.list
      r.should eq [v, @ps]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ps2 = FactoryGirl.create :provider_status
        @pd2 = FactoryGirl.create :provider, :provider_status_id => @ps2.id, :name => "5"
        @ps3 = FactoryGirl.create :provider_status
        @pd3 = FactoryGirl.create :provider, :provider_status_id => @ps3.id, :name => "4"
        @ps4 = FactoryGirl.create :provider_status
        @pd4 = FactoryGirl.create :provider, :provider_status_id => @ps4.id, :name => "3"
        @ps5 = FactoryGirl.create :provider_status
        @pd5 = FactoryGirl.create :provider, :provider_status_id => @ps5.id, :name => "2"
        ProviderStatus.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        r = ProviderStatus.list
        r.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #管理名で並んでいる
        r = ProviderStatus.list(1)
        r.should eq [@ps5, @ps4]
      end
      it 'page=2なら中間2件を返す' do
        r = ProviderStatus.list(2)
        r.should eq [@ps3, @ps2]
      end
      it 'page=3なら先頭1件を返す' do
        r = ProviderStatus.list(3)
        r.should eq [@ps]
      end
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ps2 = FactoryGirl.create :provider_status
        @pd2 = FactoryGirl.create :provider, :provider_status_id => @ps2.id, :name => "5"
        @ps3 = FactoryGirl.create :provider_status
        @pd3 = FactoryGirl.create :provider, :provider_status_id => @ps3.id, :name => "4"
        @ps4 = FactoryGirl.create :provider_status
        @pd4 = FactoryGirl.create :provider, :provider_status_id => @ps4.id, :name => "3"
        @ps5 = FactoryGirl.create :provider_status
        @pd5 = FactoryGirl.create :provider, :provider_status_id => @ps5.id, :name => "2"
        ProviderStatus.stub(:default_page_size).and_return(2)
      end
      it '件数0は全件(5件)を返す' do
        r = ProviderStatus.list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '待機中一覧取得に於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
      @pd = FactoryGirl.create :provider, :provider_status_id => @ps.id, :name => "6"
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        ProviderStatus.stub(:list_opt).with(any_args).and_return({:include => {:provider => {}} })
        ProviderStatus.should_receive(:list_opt).with(any_args).exactly(1)
        r = ProviderStatus.available_list
      end
    end
    it 'リストを返す' do
      r = ProviderStatus.available_list
      r.should eq [@ps]
    end
    it '管理名で並んでいる' do
      @ps2 = FactoryGirl.create :provider_status
      v = FactoryGirl.create :provider, :name => "0", :provider_status_id => @ps2.id
      r = ProviderStatus.available_list
      r.should eq [@ps2, @ps]
    end
    it '借受状況のトークンが設定されていない借受状況に限る' do
      @ps2 = FactoryGirl.create :provider_status, :token => 'aaaaa'
      v = FactoryGirl.create :provider, :name => "0", :provider_status_id => @ps2.id
      r = ProviderStatus.available_list
      r.should eq [@ps]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ps2 = FactoryGirl.create :provider_status
        @pd2 = FactoryGirl.create :provider, :provider_status_id => @ps2.id, :name => "5"
        @ps3 = FactoryGirl.create :provider_status
        @pd3 = FactoryGirl.create :provider, :provider_status_id => @ps3.id, :name => "4"
        @ps4 = FactoryGirl.create :provider_status
        @pd4 = FactoryGirl.create :provider, :provider_status_id => @ps4.id, :name => "3"
        @ps5 = FactoryGirl.create :provider_status
        @pd5 = FactoryGirl.create :provider, :provider_status_id => @ps5.id, :name => "2"
        ProviderStatus.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        r = ProviderStatus.available_list
        r.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #管理名で並んでいる
        r = ProviderStatus.available_list(1)
        r.should eq [@ps5, @ps4]
      end
      it 'page=2なら中間2件を返す' do
        r = ProviderStatus.available_list(2)
        r.should eq [@ps3, @ps2]
      end
      it 'page=3なら先頭1件を返す' do
        r = ProviderStatus.available_list(3)
        r.should eq [@ps]
      end
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ps2 = FactoryGirl.create :provider_status
        @pd2 = FactoryGirl.create :provider, :provider_status_id => @ps2.id, :name => "5"
        @ps3 = FactoryGirl.create :provider_status
        @pd3 = FactoryGirl.create :provider, :provider_status_id => @ps3.id, :name => "4"
        @ps4 = FactoryGirl.create :provider_status
        @pd4 = FactoryGirl.create :provider, :provider_status_id => @ps4.id, :name => "3"
        @ps5 = FactoryGirl.create :provider_status
        @pd5 = FactoryGirl.create :provider, :provider_status_id => @ps5.id, :name => "2"
        ProviderStatus.stub(:default_page_size).and_return(2)
      end
      it '件数0は全件(5件)を返す' do
        r = ProviderStatus.available_list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '一覧取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = ProviderStatus.list_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = ProviderStatus.list_opt[:include]
      r.should have(1).items
    end
    it '貸手を含んでいる' do
      r = ProviderStatus.list_opt[:include]
      r.has_key?(:provider).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
      @pd = FactoryGirl.create :provider, :provider_status_id => @ps.id
    end
    it '貸手を含んでいる' do
      r = ProviderStatus.list.to_json ProviderStatus.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('provider').should be_true
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
      @pd = FactoryGirl.create :provider, :provider_status_id => @ps.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        ProviderStatus.stub(:show_opt).with(any_args).and_return({})
        ProviderStatus.should_receive(:show_opt).with(any_args).exactly(1)
        r = ProviderStatus.show @ps.id, @admin
      end
      it '閲覧許可を問い合わせている' do
        ProviderStatus.any_instance.stub(:visible?).with(any_args).and_return(true)
        ProviderStatus.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = ProviderStatus.show @ps.id, @admin
      end
    end
    it '指定の借受状況を返す' do
      r = ProviderStatus.show @ps.id, @admin
      r.should eq @ps
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        ProviderStatus.any_instance.stub(:visible?).and_return(false)
        lambda{
          ProviderStatus.show @ps.id, @admin
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しない借受状況を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          ProviderStatus.show 110, @admin
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '編集取得に於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
      @pd = FactoryGirl.create :provider, :provider_status_id => @ps.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        ProviderStatus.stub(:show_opt).with(any_args).and_return({})
        ProviderStatus.should_receive(:show_opt).with(any_args).exactly(1)
        r = ProviderStatus.edit @ps.id, @admin
      end
      it '所持判定を問い合わせている' do
        ProviderStatus.any_instance.stub(:own?).with(any_args).and_return(true)
        ProviderStatus.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = ProviderStatus.edit @ps.id, @admin
      end
    end
    it '指定の借受状況を返す' do
      ProviderStatus.any_instance.stub(:own?).and_return(true)
      r = ProviderStatus.edit @ps.id, @admin
      r.should eq @ps
    end
    context '権限がなかったとき' do
      it '403Forbidden例外を返す' do
        ProviderStatus.any_instance.stub(:own?).and_return(false)
        lambda{
          ProviderStatus.edit @ps.id, @admin
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しない借受状況を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          ProviderStatus.edit 110, @admin
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = ProviderStatus.show_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = ProviderStatus.show_opt[:include]
      r.should have(1).items
    end
    it '貸手を含んでいる' do
      r = ProviderStatus.show_opt[:include]
      r.has_key?(:provider).should be_true
    end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @ps = FactoryGirl.create :provider_status
      @pd = FactoryGirl.create :provider, :provider_status_id => @ps.id
    end
    it '貸手を含んでいる' do
      r = ProviderStatus.show(@ps.id, @admin).to_json ProviderStatus.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('provider').should be_true
    end
  end
  
end
